/*
 *  Copyright 2009 Citrix Systems, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  For clarity, the licensor of this program does not intend that a
 *  "derivative work" include code which compiles header information from
 *  this program. 
 *
 */

#ifndef _INPUTVSC_API_H_
#define _INPUTVSC_API_H_

#include "VmbusApi.h"

//
// Defines
//
#define INPUTVSC_SEND_RING_BUFFER_SIZE		10*PAGE_SIZE
#define INPUTVSC_RECV_RING_BUFFER_SIZE		10*PAGE_SIZE


//
// Data types
//

typedef struct {
	USHORT  VendorID;
    USHORT  ProductID;
    USHORT  VersionNumber;
	char	Name[128];
} INPUT_DEVICE_INFO;

typedef int (*PFN_ON_OPEN)(DEVICE_OBJECT *Device);
typedef int (*PFN_ON_CLOSE)(DEVICE_OBJECT *Device);

typedef void (*PFN_ON_DEVICEINFO)(DEVICE_OBJECT *dev, INPUT_DEVICE_INFO* info);
typedef void (*PFN_ON_INPUTREPORT)(DEVICE_OBJECT *dev, void* packet, UINT32 len);
typedef void (*PFN_ON_REPORTDESC)(DEVICE_OBJECT *dev, void* packet, UINT32 len);


// Represents the input vsc driver
typedef struct _INPUTVSC_DRIVER_OBJECT {
	DRIVER_OBJECT				Base; // Must be the first field
	

	// This is set by the caller to allow us to callback when we receive a packet
	// from the "wire"
	PFN_ON_DEVICEINFO			OnDeviceInfo;
	PFN_ON_INPUTREPORT			OnInputReport;
	PFN_ON_REPORTDESC			OnReportDescriptor;

	// Specific to this driver
	PFN_ON_OPEN					OnOpen;
	PFN_ON_CLOSE				OnClose;

	void*						Context;
} INPUTVSC_DRIVER_OBJECT;


//
// Interface
//
int 
InputVscInitialize(
	DRIVER_OBJECT* drv
	);

#endif // _INPUTVSC_API_H_
