/*++

Copyright (c) 2008, Microsoft. All rights reserved.
Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
	Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
	Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution. 
	Neither the name of the Microsoft nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission. 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--*/

/*
**
** Open-source vmbus header file that defines the open-source vmbus module.
** This interface is used by open-source component and can include Linux open-source
** kernel headers
**
*/

#ifndef _VMBUS_H_
#define _VMBUS_H_

#include <linux/device.h>

#include "VmbusApi.h"	// public header file for our closed-source vmbus module

//
// Data types
//

typedef int (*PFN_DRIVERINITIALIZE)(DRIVER_OBJECT *drv);
typedef int (*PFN_DRIVEREXIT)(DRIVER_OBJECT *drv);

struct driver_context {
	GUID					class_id;

	struct device_driver	driver;

	// Use these methods instead of the struct device_driver so 2.6 kernel stops complaining
	int (*probe)(struct device *);
	int (*remove)(struct device *);
	void (*shutdown)(struct device *);
};

struct device_context {
	GUID					class_id;
	GUID					device_id;
	int						probe_error;
	struct work_struct		probe_failed_work_item;
	struct device			device;
	DEVICE_OBJECT			device_obj;
};


//
// Global
//

//
// Inlines
//
static inline struct device_context *to_device_context(DEVICE_OBJECT *device_obj)
{
	return container_of(device_obj, struct device_context, device_obj);
}

static inline struct device_context *device_to_device_context(struct device *device)
{
	return container_of(device, struct device_context, device);
}

static inline struct driver_context *driver_to_driver_context(struct device_driver *driver)
{
	return container_of(driver, struct driver_context, driver);
}

#if defined(KERNEL_2_6_5)
static inline void* kzalloc(int size, int flags)
{
	void *p;
	p = kmalloc(size, flags);
	if (p) memset(p, 0, size);

	return p;
}
#endif // KERNEL_2_6_5

//
// Vmbus interface
//
void 
vmbus_child_driver_register(
	struct driver_context* driver_ctx
	);

void
vmbus_child_driver_unregister(
	struct driver_context *driver_ctx
	);

void
vmbus_get_interface(
	VMBUS_CHANNEL_INTERFACE *interface
	);

#endif // _VMBUS_H_
